@@ -68,12 +68,6 @@ enum
   LAST_SIGNAL
 };
 
-static const GstElementDetails gst_cmml_dec_details =
-GST_ELEMENT_DETAILS ("CMML stream decoder",
-    "Codec/Decoder",
-    "Decodes CMML streams",
-    "Alessandro Decina <alessandro@nnva.org>");
-
 static GstStaticPadTemplate gst_cmml_dec_src_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -131,7 +125,9 @@ gst_cmml_dec_base_init (gpointer g_class)
       gst_static_pad_template_get (&gst_cmml_dec_sink_factory));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_cmml_dec_src_factory));
-  gst_element_class_set_details (element_class, &gst_cmml_dec_details);
+  gst_element_class_set_details_simple (element_class, "CMML stream decoder",
+      "Codec/Decoder",
+      "Decodes CMML streams", "Alessandro Decina <alessandro@nnva.org>");
 }
 
 static void
@@ -150,7 +146,7 @@ gst_cmml_dec_class_init (GstCmmlDecClass * dec_class)
           "Wait clip end time",
           "Send a tag for a clip when the clip ends, setting its end-time. "
           "Use when you need to know both clip's start-time and end-time.",
-          FALSE, G_PARAM_READWRITE));
+          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -515,7 +511,7 @@ gst_cmml_dec_parse_first_header (GstCmmlDec * dec, GstBuffer * buffer)
   /* if there is a processing instruction, gst_cmml_dec_parse_preamble
    * will be triggered. Otherwise we need to call it manually.
    */
-  if (!GST_FLOW_IS_FATAL (dec->flow_return) && !dec->sent_root) {
+  if (dec->flow_return == GST_FLOW_OK && !dec->sent_root) {
     guchar *preamble = (guchar *) g_strndup ((gchar *) GST_BUFFER_DATA (buffer),
         GST_BUFFER_SIZE (buffer));
 
@@ -559,9 +555,7 @@ gst_cmml_dec_parse_preamble (GstCmmlDec * dec, guchar * preamble,
     dec->flow_return = gst_pad_push (dec->srcpad, buffer);
   }
 
-  if (dec->flow_return == GST_FLOW_NOT_LINKED)
-    dec->flow_return = GST_FLOW_OK;     /* Ignore NOT_LINKED */
-  if (!GST_FLOW_IS_FATAL (dec->flow_return)) {
+  if (dec->flow_return == GST_FLOW_OK) {
     GST_INFO_OBJECT (dec, "preamble parsed");
   }
 
