@@ -345,7 +345,7 @@ gst_jpegenc_getcaps (GstPad * pad)
   /* we want to proxy properties like width, height and framerate from the
      other end of the element */
 
-  othercaps = gst_pad_get_allowed_caps (jpegenc->srcpad);
+  othercaps = gst_pad_peer_get_caps_reffed (jpegenc->srcpad);
   if (othercaps == NULL ||
       gst_caps_is_empty (othercaps) || gst_caps_is_any (othercaps)) {
     caps = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
@@ -520,11 +520,12 @@ gst_jpegenc_resync (GstJpegEnc * jpegenc)
         jpegenc->h_samp[i], jpegenc->v_samp[i]);
     jpegenc->cinfo.comp_info[i].h_samp_factor = jpegenc->h_samp[i];
     jpegenc->cinfo.comp_info[i].v_samp_factor = jpegenc->v_samp[i];
-    jpegenc->line[i] = g_realloc (jpegenc->line[i],
-        jpegenc->v_max_samp * DCTSIZE * sizeof (char *));
+    g_free (jpegenc->line[i]);
+    jpegenc->line[i] = g_new (guchar *, jpegenc->v_max_samp * DCTSIZE);
     if (!jpegenc->planar) {
       for (j = 0; j < jpegenc->v_max_samp * DCTSIZE; j++) {
-        jpegenc->row[i][j] = g_realloc (jpegenc->row[i][j], width);
+        g_free (jpegenc->row[i][j]);
+        jpegenc->row[i][j] = g_malloc (width);
         jpegenc->line[i][j] = jpegenc->row[i][j];
       }
     }
@@ -546,7 +547,7 @@ gst_jpegenc_chain (GstPad * pad, GstBuffer * buf)
   GstJpegEnc *jpegenc;
   guchar *data;
   gulong size;
-  guint height, width;
+  guint height;
   guchar *base[3], *end[3];
   gint i, j, k;
 
@@ -571,7 +572,6 @@ gst_jpegenc_chain (GstPad * pad, GstBuffer * buf)
   gst_buffer_copy_metadata (jpegenc->output_buffer, buf,
       GST_BUFFER_COPY_TIMESTAMPS);
 
-  width = jpegenc->width;
   height = jpegenc->height;
 
   for (i = 0; i < jpegenc->channels; i++) {
