@@ -66,7 +66,7 @@
 
 #define SEND_COMMAND(src, command)          \
 G_STMT_START {                              \
-  int _res; unsigned char c; c = command;   \
+  int G_GNUC_UNUSED _res; unsigned char c; c = command;   \
   _res = write (WRITE_SOCKET(src), &c, 1);  \
 } G_STMT_END
 
@@ -130,6 +130,8 @@ static void gst_dv1394src_get_property (GObject * object, guint prop_id,
 static void gst_dv1394src_dispose (GObject * object);
 
 static GstClock *gst_dv1394src_provide_clock (GstElement * element);
+static GstStateChangeReturn gst_dv1394_src_change_state (GstElement * element,
+    GstStateChange transition);
 
 static gboolean gst_dv1394src_start (GstBaseSrc * bsrc);
 static gboolean gst_dv1394src_stop (GstBaseSrc * bsrc);
@@ -197,6 +199,7 @@ gst_dv1394src_class_init (GstDV1394SrcClass * klass)
   gobject_class->dispose = gst_dv1394src_dispose;
 
   gstelement_class->provide_clock = gst_dv1394src_provide_clock;
+  gstelement_class->change_state = gst_dv1394_src_change_state;
 
   gst_dv1394src_signals[SIGNAL_FRAME_DROPPED] =
       g_signal_new ("frame-dropped", G_TYPE_FROM_CLASS (klass),
@@ -205,29 +208,30 @@ gst_dv1394src_class_init (GstDV1394SrcClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PORT,
       g_param_spec_int ("port", "Port", "Port number (-1 automatic)",
-          -1, 16, DEFAULT_PORT, G_PARAM_READWRITE));
+          -1, 16, DEFAULT_PORT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_CHANNEL,
       g_param_spec_int ("channel", "Channel", "Channel number for listening",
-          0, 64, DEFAULT_CHANNEL, G_PARAM_READWRITE));
+          0, 64, DEFAULT_CHANNEL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_CONSECUTIVE,
       g_param_spec_int ("consecutive", "consecutive frames",
           "send n consecutive frames after skipping", 1, G_MAXINT,
-          DEFAULT_CONSECUTIVE, G_PARAM_READWRITE));
+          DEFAULT_CONSECUTIVE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SKIP,
       g_param_spec_int ("skip", "skip frames", "skip n frames",
-          0, G_MAXINT, DEFAULT_SKIP, G_PARAM_READWRITE));
+          0, G_MAXINT, DEFAULT_SKIP,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_DROP_INCOMPLETE,
-      g_param_spec_boolean ("drop_incomplete", "drop_incomplete",
+      g_param_spec_boolean ("drop-incomplete", "drop incomplete",
           "drop incomplete frames", DEFAULT_DROP_INCOMPLETE,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_USE_AVC,
       g_param_spec_boolean ("use-avc", "Use AV/C", "Use AV/C VTR control",
-          DEFAULT_USE_AVC, G_PARAM_READWRITE));
+          DEFAULT_USE_AVC, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_GUID,
       g_param_spec_uint64 ("guid", "GUID",
           "select one of multiple DV devices by its GUID. use a hexadecimal "
           "like 0xhhhhhhhhhhhhhhhh. (0 = no guid)", 0, G_MAXUINT64,
-          DEFAULT_GUID, G_PARAM_READWRITE));
+          DEFAULT_GUID, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstDV1394Src:device-name
    *
@@ -237,7 +241,8 @@ gst_dv1394src_class_init (GstDV1394SrcClass * klass)
    **/
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_DEVICE_NAME,
       g_param_spec_string ("device-name", "device name",
-          "user-friendly name of the device", "Default", G_PARAM_READABLE));
+          "user-friendly name of the device", "Default",
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   gstbasesrc_class->negotiate = NULL;
   gstbasesrc_class->start = gst_dv1394src_start;
@@ -381,6 +386,39 @@ gst_dv1394src_provide_clock (GstElement * element)
   return GST_CLOCK_CAST (gst_object_ref (dv1394src->provided_clock));
 }
 
+static GstStateChangeReturn
+gst_dv1394_src_change_state (GstElement * element, GstStateChange transition)
+{
+  GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
+  GstDV1394Src *src = GST_DV1394SRC (element);
+
+  switch (transition) {
+    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+      gst_element_post_message (element,
+          gst_message_new_clock_lost (GST_OBJECT_CAST (element),
+              GST_CLOCK_CAST (src->provided_clock)));
+      break;
+    default:
+      break;
+  }
+
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+  if (ret == GST_STATE_CHANGE_FAILURE)
+    return ret;
+
+  switch (transition) {
+    case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
+      gst_element_post_message (element,
+          gst_message_new_clock_provide (GST_OBJECT_CAST (element),
+              GST_CLOCK_CAST (src->provided_clock), TRUE));
+      break;
+    default:
+      break;
+  }
+
+  return ret;
+}
+
 #ifdef HAVE_LIBIEC61883
 static GstDV1394Src *
 gst_dv1394src_from_raw1394handle (raw1394handle_t handle)
