@@ -62,7 +62,7 @@
 
 #define SEND_COMMAND(src, command)          \
 G_STMT_START {                              \
-  int _res; unsigned char c; c = command;   \
+  int G_GNUC_UNUSED _res; unsigned char c; c = command;   \
   _res = write (WRITE_SOCKET(src), &c, 1);  \
 } G_STMT_END
 
@@ -166,18 +166,18 @@ gst_hdv1394src_class_init (GstHDV1394SrcClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PORT,
       g_param_spec_int ("port", "Port", "Port number (-1 automatic)",
-          -1, 16, DEFAULT_PORT, G_PARAM_READWRITE));
+          -1, 16, DEFAULT_PORT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_CHANNEL,
       g_param_spec_int ("channel", "Channel", "Channel number for listening",
-          0, 64, DEFAULT_CHANNEL, G_PARAM_READWRITE));
+          0, 64, DEFAULT_CHANNEL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_USE_AVC,
       g_param_spec_boolean ("use-avc", "Use AV/C", "Use AV/C VTR control",
-          DEFAULT_USE_AVC, G_PARAM_READWRITE));
+          DEFAULT_USE_AVC, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_GUID,
       g_param_spec_uint64 ("guid", "GUID",
           "select one of multiple DV devices by its GUID. use a hexadecimal "
           "like 0xhhhhhhhhhhhhhhhh. (0 = no guid)", 0, G_MAXUINT64,
-          DEFAULT_GUID, G_PARAM_READWRITE));
+          DEFAULT_GUID, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstHDV1394Src:device-name
    *
@@ -187,7 +187,8 @@ gst_hdv1394src_class_init (GstHDV1394SrcClass * klass)
    **/
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_DEVICE_NAME,
       g_param_spec_string ("device-name", "device name",
-          "user-friendly name of the device", "Default", G_PARAM_READABLE));
+          "user-friendly name of the device", "Default",
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
   gstbasesrc_class->negotiate = NULL;
   gstbasesrc_class->start = gst_hdv1394src_start;
