@@ -35,6 +35,10 @@
 
 #include "gstdeinterlacemethod.h"
 #include <string.h>
+#ifdef HAVE_ORC
+#include <orc/orc.h>
+#endif
+#include "tvtime.h"
 
 #define GST_TYPE_DEINTERLACE_METHOD_VFIR	(gst_deinterlace_method_vfir_get_type ())
 #define GST_IS_DEINTERLACE_METHOD_VFIR(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_DEINTERLACE_METHOD_VFIR))
@@ -57,29 +61,14 @@ typedef GstDeinterlaceSimpleMethodClass GstDeinterlaceMethodVFIRClass;
  * filter taps here are: [-1 4 2 4 -1].
  */
 
-/**
+ /*
   * C implementation.
   */
 static inline void
 deinterlace_c (guint8 * dst, const guint8 * lum_m4, const guint8 * lum_m3,
     const guint8 * lum_m2, const guint8 * lum_m1, const guint8 * lum, gint size)
 {
-  gint sum;
-
-  for (; size >= 0; size--) {
-    sum = -lum_m4[0];
-    sum += lum_m3[0] << 2;
-    sum += lum_m2[0] << 1;
-    sum += lum_m1[0] << 2;
-    sum += -lum[0];
-    dst[0] = (sum + 4) >> 3;    // This needs to be clipped at 0 and 255: cm[(sum + 4) >> 3];
-    lum_m4++;
-    lum_m3++;
-    lum_m2++;
-    lum_m1++;
-    lum++;
-    dst++;
-  }
+  deinterlace_line_vfir (dst, lum_m4, lum_m3, lum_m2, lum_m1, lum, size);
 }
 
 static void
@@ -255,7 +244,8 @@ gst_deinterlace_method_vfir_class_init (GstDeinterlaceMethodVFIRClass * klass)
   GstDeinterlaceSimpleMethodClass *dism_class =
       (GstDeinterlaceSimpleMethodClass *) klass;
 #ifdef BUILD_X86_ASM
-  guint cpu_flags = oil_cpu_get_flags ();
+  guint cpu_flags =
+      orc_target_get_default_flags (orc_target_get_by_name ("mmx"));
 #endif
 
   dim_class->fields_required = 2;
@@ -264,11 +254,13 @@ gst_deinterlace_method_vfir_class_init (GstDeinterlaceMethodVFIRClass * klass)
   dim_class->latency = 0;
 
 #ifdef BUILD_X86_ASM
-  if (cpu_flags & OIL_IMPL_FLAG_MMX) {
+  if (cpu_flags & ORC_TARGET_MMX_MMX) {
     dism_class->interpolate_scanline_ayuv = deinterlace_line_packed_mmx;
     dism_class->interpolate_scanline_yuy2 = deinterlace_line_packed_mmx;
     dism_class->interpolate_scanline_yvyu = deinterlace_line_packed_mmx;
     dism_class->interpolate_scanline_uyvy = deinterlace_line_packed_mmx;
+    dism_class->interpolate_scanline_nv12 = deinterlace_line_packed_mmx;
+    dism_class->interpolate_scanline_nv21 = deinterlace_line_packed_mmx;
     dism_class->interpolate_scanline_argb = deinterlace_line_packed_mmx;
     dism_class->interpolate_scanline_abgr = deinterlace_line_packed_mmx;
     dism_class->interpolate_scanline_rgba = deinterlace_line_packed_mmx;
@@ -283,6 +275,8 @@ gst_deinterlace_method_vfir_class_init (GstDeinterlaceMethodVFIRClass * klass)
     dism_class->interpolate_scanline_yvyu = deinterlace_line_packed_c;
     dism_class->interpolate_scanline_uyvy = deinterlace_line_packed_c;
     dism_class->interpolate_scanline_ayuv = deinterlace_line_packed_c;
+    dism_class->interpolate_scanline_nv12 = deinterlace_line_packed_c;
+    dism_class->interpolate_scanline_nv21 = deinterlace_line_packed_c;
     dism_class->interpolate_scanline_argb = deinterlace_line_packed_c;
     dism_class->interpolate_scanline_abgr = deinterlace_line_packed_c;
     dism_class->interpolate_scanline_rgba = deinterlace_line_packed_c;
@@ -298,6 +292,8 @@ gst_deinterlace_method_vfir_class_init (GstDeinterlaceMethodVFIRClass * klass)
   dism_class->interpolate_scanline_yuy2 = deinterlace_line_packed_c;
   dism_class->interpolate_scanline_yvyu = deinterlace_line_packed_c;
   dism_class->interpolate_scanline_uyvy = deinterlace_line_packed_c;
+  dism_class->interpolate_scanline_nv12 = deinterlace_line_packed_c;
+  dism_class->interpolate_scanline_nv21 = deinterlace_line_packed_c;
   dism_class->interpolate_scanline_argb = deinterlace_line_packed_c;
   dism_class->interpolate_scanline_abgr = deinterlace_line_packed_c;
   dism_class->interpolate_scanline_rgba = deinterlace_line_packed_c;
