@@ -92,6 +92,8 @@ typedef struct _GstMatroskaDemux {
 
   /* a cue (index) table */
   GArray                  *index;
+  /* cluster positions (optional) */
+  GArray                  *clusters;
 
   /* timescale in the file */
   guint64                  time_scale;
@@ -114,6 +116,7 @@ typedef struct _GstMatroskaDemux {
   GstClockTime             cluster_time;
   guint64                  cluster_offset;
   guint64                  first_cluster_offset;
+  guint64                  next_cluster_offset;
 
   /* push based mode usual suspects */
   GstAdapter              *adapter;
