@@ -106,7 +106,7 @@ GST_BOILERPLATE (GstRTPDVPay, gst_rtp_dv_pay, GstBaseRTPPayload,
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_dv_pay_src_template));
   gst_element_class_set_details_simple (element_class, "RTP DV Payloader",
-      "Codec/Payloader/Network",
+      "Codec/Payloader/Network/RTP",
       "Payloads DV into RTP packets (RFC 3189)",
       "Marcel Moreaux <marcelm@spacelabs.nl>, Wim Taymans <wim.taymans@gmail.com>");
 }
@@ -371,5 +371,5 @@ gboolean
 gst_rtp_dv_pay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpdvpay",
-      GST_RANK_NONE, GST_TYPE_RTP_DV_PAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_DV_PAY);
 }
