@@ -44,6 +44,7 @@ struct _GstRtpMP4ADepay
   GstBaseRTPDepayload depayload;
   GstAdapter *adapter;
   guint8 numSubFrames;
+  guint frame_len;
 };
 
 struct _GstRtpMP4ADepayClass
@@ -51,6 +52,8 @@ struct _GstRtpMP4ADepayClass
   GstBaseRTPDepayloadClass parent_class;
 };
 
+GType gst_rtp_mp4a_depay_get_type (void);
+
 gboolean gst_rtp_mp4a_depay_plugin_init (GstPlugin * plugin);
 
 G_END_DECLS
