@@ -78,7 +78,7 @@ gst_rtp_mp4v_depay_base_init (gpointer klass)
       gst_static_pad_template_get (&gst_rtp_mp4v_depay_sink_template));
 
   gst_element_class_set_details_simple (element_class,
-      "RTP MPEG4 video depayloader", "Codec/Depayloader/Network",
+      "RTP MPEG4 video depayloader", "Codec/Depayloader/Network/RTP",
       "Extracts MPEG4 video from RTP packets (RFC 3016)",
       "Wim Taymans <wim.taymans@gmail.com>");
 }
@@ -227,5 +227,5 @@ gboolean
 gst_rtp_mp4v_depay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpmp4vdepay",
-      GST_RANK_MARGINAL, GST_TYPE_RTP_MP4V_DEPAY);
+      GST_RANK_SECONDARY, GST_TYPE_RTP_MP4V_DEPAY);
 }
