@@ -21,10 +21,10 @@
  */
 
 #include <unistd.h>
+#include <math.h>
 
 #include <gst/check/gstcheck.h>
 
-GList *buffers = NULL;
 gboolean have_eos = FALSE;
 
 /* For ease of programming we use globals to keep refs for our floating
@@ -63,8 +63,8 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     );
 
 /* takes over reference for outcaps */
-GstElement *
-setup_level ()
+static GstElement *
+setup_level (void)
 {
   GstElement *level;
 
@@ -78,7 +78,7 @@ setup_level ()
   return level;
 }
 
-void
+static void
 cleanup_level (GstElement * level)
 {
   GST_DEBUG ("cleanup_level");
@@ -152,7 +152,7 @@ GST_START_TEST (test_int16)
 
   /* block wave of half amplitude has -5.94 dB for rms, peak and decay */
   for (i = 0; i < 2; ++i) {
-    gchar *fields[3] = { "rms", "peak", "decay" };
+    const gchar *fields[3] = { "rms", "peak", "decay" };
     for (j = 0; j < 3; ++j) {
       list = gst_structure_get_value (structure, fields[j]);
       value = gst_value_list_get_value (list, i);
@@ -200,7 +200,7 @@ GST_START_TEST (test_int16_panned)
   const GValue *list, *value;
   GstClockTime endtime;
   gdouble dB;
-  gchar *fields[3] = { "rms", "peak", "decay" };
+  const gchar *fields[3] = { "rms", "peak", "decay" };
 
   level = setup_level ();
   g_object_set (level, "message", TRUE, "interval", GST_SECOND / 10, NULL);
@@ -255,7 +255,11 @@ GST_START_TEST (test_int16_panned)
     value = gst_value_list_get_value (list, 0);
     dB = g_value_get_double (value);
     GST_DEBUG ("%s[0] is %lf", fields[j], dB);
-    fail_if (!isinf (dB));
+#ifdef HAVE_ISINF
+    fail_unless (isinf (dB));
+#elif defined (HAVE_FPCLASS)
+    fail_unless (fpclass (dB) == FP_NINF);
+#endif
   }
   /* block wave of half amplitude has -5.94 dB for rms, peak and decay */
   for (j = 0; j < 3; ++j) {
@@ -291,7 +295,7 @@ GST_START_TEST (test_int16_panned)
 
 GST_END_TEST;
 
-Suite *
+static Suite *
 level_suite (void)
 {
   Suite *s = suite_create ("level");
